require 'rexml/document'
require "amrita2/core"
include Amrita2

tmpl = REXML::Document.new <<END
<table border='1'>
  <tr><th></th><th></th></tr>
  <tr class='detail1'><td></td><td></td></tr>
  <tr class='detail2'><td></td><td></td></tr>
</table>
END

include HtmlTag
spec = Amrita2::define_template_spec(:direct_define=>true) {
  table(:matcher=>TABLE) {
    header(:matcher=>TR) {
      item1(:matcher=>TH)
      item2(:matcher=>TH)
    }
    detail(:matcher=>TR, :match_many=>true) {
      name(:matcher=>TD)
      author(:matcher=>TD)
    }
  }
}

data = [
  [ "Ruby", "matz" ],
  [ "perl", "Larry Wall" ],
  [ "python", "Guido van Rossum" ]
] 

spec.create_template(tmpl).expand_template(STDOUT) do |m|
  m.table do |mt|
    mt.header do |mh|
      mh.item1('name')
      mh.item2('author')
    end
    data.each_with_index do |data, i|
      mt.detail(i) do |md|
        md.name(data[0])
        md.author(data[1])
      end
    end
  end
end

puts

__END__
<table border='1'>
  <tr><th>name</th><th>author</th></tr>
  <tr class='detail1'><td>Ruby</td><td>matz</td></tr>
  <tr class='detail2'><td>perl</td><td>Larry Wall</td></tr>
  <tr class='detail1'><td>python</td><td>Guido van Rossum</td></tr>
</table>

