require 'test/unit'
require 'test/unit/testsuite'
require 'test/unit/ui/console/testrunner'
require 'amrita2/test'
require 'rexml/document'

# ruby -I~/amrita2/lib ~/amrita2/test/test_test.rb   |ruby -pe '$_.sub!(/^ */, "") ' 

module Amrita2TestTest
  include Amrita2
  include Amrita2::Core

  class TestTracer < Test::Unit::TestCase
    include Amrita2
    include Amrita2::HtmlTag

    def test_simple
      spec = Amrita2::define_template_spec do 
        dynamic_element(:aaa)
      end
      t = spec.create_dummy_template
      t.expand_template(nil) do |m|
        m.aaa('123')
      end

      assert_equal('123', t[:aaa])
    end

    def test_nest
      spec = Amrita2::define_template_spec do 
        dynamic_element(:aaa) do
          dynamic_element(:bbb)
        end
      end
      t = spec.create_dummy_template
      t.expand_template(nil) do |m|
        m.aaa do |ma|
          ma.bbb('456')
        end
      end

      assert_equal('456', t[:aaa][:bbb])
    end

    def test_nest2
      spec = Amrita2::define_template_spec do 
        dynamic_element(:aaa) do
          dynamic_element(:bbb)
          dynamic_element(:ccc)
        end
      end
      t = spec.create_dummy_template
      ret = t.expand_template(nil) do |m|
        m.aaa do |ma|
          ma.bbb('123')
          ma.ccc('456')
        end
        m.aaa do |ma|
          ma.bbb('abc')
          ma.ccc('efg')
        end
      end
      assert_equal('123', t[:aaa][0][:bbb])
      assert_equal('456', t[:aaa][0][:ccc])
      assert_equal('abc', t[:aaa][1][:bbb])
      assert_equal('efg', t[:aaa][1][:ccc])
    end

    def test_nest3
      spec = Amrita2::define_template_spec do 
        dynamic_element(:aaa) do
          dynamic_element(:bbb)
          dynamic_element(:ccc)
        end
        dynamic_element(:xxx) do
          dynamic_element(:yyy) do
            dynamic_element(:zzz) 
          end
        end
      end
      t = spec.create_dummy_template
      t.expand_template(nil) do |m|
        m.aaa do |ma|
          ma.bbb('1')
          ma.ccc('2')
          ma.ccc('3')
          ma.ccc('4')
        end
        m.xxx do |mx|
          mx.yyy do |my|
            my.zzz('z')
          end
        end
      end

      assert_equal('1', t[:aaa][:bbb])
      assert_equal(3, t[:aaa][:ccc].size)
      assert_equal('2', t[:aaa][:ccc][0])
      assert_equal('3', t[:aaa][:ccc][1])
      assert_equal('4', t[:aaa][:ccc][2])
      assert_equal('z', t[:xxx][:yyy][:zzz])
    end

    def test_replace_attr
      spec = Amrita2::define_template_spec do 
        dynamic_element(:aaa, :replace_attr=>true)
      end

      t = spec.create_dummy_template

      t.expand_template() do |m|
        m.aaa('xxx', :class=>'yyy')
      end
      assert_equal('xxx', t[:aaa])
      assert_equal('yyy', t[:aaa_attr][:class])
    end

    S1 = Struct.new(:language, :template, :no)

    def test_table
      spec = Amrita2::define_template_spec do 
        dynamic_element(:table,:matcher=>TagMatcher.new(:table)) {
          dynamic_element(:line1, :expand_by_member=>true, :delete_id=>true) {
            dynamic_element(:no, :matcher=>TagMatcher.new(:td))
            dynamic_element(:language, :matcher=>TagMatcher.new(:td))
            dynamic_element(:template, :matcher=>TagMatcher.new(:td))
          }
          dynamic_element(:line2, :expand_by_member=>true, :output_stream=>:line1, :delete_id=>true) {
            dynamic_element(:no, :matcher=>TagMatcher.new(:td))
            dynamic_element(:language, :matcher=>TagMatcher.new(:td))
            dynamic_element(:template, :matcher=>TagMatcher.new(:td))
          }
        }
      end

      t = spec.create_dummy_template
      data = [
        S1.new('ruby', 'amrita2'),
        S1.new('perl', 'HTMLtemplate'),
        S1.new('python', 'TAL'),
        S1.new('java', 'JSF')
      ]
      t.expand_template do |m|
        m.table do |mt|
          data.each_with_index do |d, i|
            d.no = i+1
            if i%2 == 0
              mt.line1(d)
            else
              mt.line2(d)
            end
          end
        end
      end
      assert_equal('ruby', t[:table][:line1][0].language)
      assert_equal('amrita2', t[:table][:line1][0].template)
      assert_equal('perl', t[:table][:line1][1].language)
      assert_equal('HTMLtemplate', t[:table][:line1][1].template)
      assert_equal('python', t[:table][:line1][2].language)
      assert_equal('TAL', t[:table][:line1][2].template)
      assert_equal('java', t[:table][:line1][3].language)
      assert_equal('JSF', t[:table][:line1][3].template)
    end
  end
end

if __FILE__ == $0
  include Amrita2TestTest
  cname = ARGV.shift
  if cname
    method = ARGV.shift
    if method
      eval "Test::Unit::UI::Console::TestRunner.run(#{cname}.new(\"#{method}\"))"
    else
      cls = eval(cname)
      Test::Unit::UI::Console::TestRunner.run(cls)
    end
  else
    suite = Test::Unit::TestSuite.new
    suite << TestTracer.suite
    Test::Unit::UI::Console::TestRunner.run(suite)
  end
end
